#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2014 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Income activity implementation
"""

from persistent import Persistent
from zope.component import adapter
from zope.container.btree import BTreeContainer
from zope.container.contained import Contained
from zope.interface import implementer
from zope.interface import implements
from zope.schema.interfaces import IIterableVocabulary

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.utils import TitledContainerItemVocabulary

from schooltool.fee import interfaces


INCOME_GENERATING_ACTIVITY_KEY = 'schooltool.fee.income_generating_activity'
INCOME_GENERATING_ACTIVITY_CATEGORY_KEY = 'schooltool.fee.income_generating_activity_category'


class IncomeGeneratingActivityContainer(BTreeContainer):

    implements(interfaces.IIncomeGeneratingActivityContainer)


class IncomeGeneratingActivity(BTreeContainer):

    implements(interfaces.IIncomeGeneratingActivity)

    title = None
    description = None
    closed = False
    projected_income = None

    @property
    def current_income(self):
        return sum([item.balance_amount for item in self.values()])


@implementer(interfaces.IIncomeGeneratingActivityContainer)
@adapter(ISchoolToolApplication)
def getApplicationIncomeGeneratingActivityContainer(app):
    try:
        return app[INCOME_GENERATING_ACTIVITY_KEY]
    except (KeyError,):
        container = IncomeGeneratingActivityContainer()
        app[INCOME_GENERATING_ACTIVITY_KEY] = container
        return app[INCOME_GENERATING_ACTIVITY_KEY]


class IncomeGeneratingActivityCategoryContainer(BTreeContainer):

    implements(interfaces.IIncomeGeneratingActivityCategoryContainer)

    default_id = None

    @property
    def default(self):
        if self.default_id is None:
            return None
        return self.get(self.default_id, None)

    @default.setter
    def default(self, category):
        if category is None:
            self.default_id = None
            return
        key = category.__name__
        if key not in self:
            raise KeyError(key)
        self.default_id = key


class IncomeGeneratingActivityCategory(Persistent, Contained):

    implements(interfaces.IIncomeGeneratingActivityCategory)

    title = None


@implementer(interfaces.IIncomeGeneratingActivityCategoryContainer)
@adapter(ISchoolToolApplication)
def getApplicationIncomeGeneratingActivityCategoryContainer(app):
    try:
        return app[INCOME_GENERATING_ACTIVITY_CATEGORY_KEY]
    except (KeyError,):
        container = IncomeGeneratingActivityCategoryContainer()
        app[INCOME_GENERATING_ACTIVITY_CATEGORY_KEY] = container
        return app[INCOME_GENERATING_ACTIVITY_CATEGORY_KEY]


class IncomeGeneratingActivityCategoryVocabulary(TitledContainerItemVocabulary):

    implements(IIterableVocabulary)

    @property
    def container(self):
        app = ISchoolToolApplication(None)
        return interfaces.IIncomeGeneratingActivityCategoryContainer(app)


def IncomeGeneratingActivityCategoryVocabularyFactory():
    return IncomeGeneratingActivityCategoryVocabulary
