#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2014 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Fee related interfaces
"""

from zope.container.constraints import contains
from zope.interface import Interface
from zope.schema import Bool
from zope.schema import Choice
from zope.schema import Date
from zope.schema import Decimal
from zope.schema import Object
from zope.schema import Text
from zope.schema import TextLine

from schooltool.common import SchoolToolMessage as STMessage

from schooltool.fee import FeeMessage as _


class IFeeContainer(Interface):

    contains('.IAmount')


class ISchoolYearFeeContainer(IFeeContainer):

    pass


class IGroupFeeContainer(IFeeContainer):

    pass


class IStreamFeeContainer(IFeeContainer):

    pass


class IAmount(Interface):

    date = Date(title=STMessage(u'Date'))

    title = TextLine(title=STMessage(u'Title'), required=False)

    description = Text(title=STMessage(u'Description'), required=False)

    amount = Decimal(title=_(u'Amount'))

    balance_amount = Decimal(title=u'Amount used in balance calculation')


class IDebit(IAmount):

    title = TextLine(title=STMessage(u'Title'))

    label = TextLine(
        title=STMessage(u'Label'),
        description=_(u'Limit to 7 characters or less'),
        max_length=7)

    last_date = Date(title=_(u'Last date'))

    master = TextLine(title=u'Master Fee ID', required=False)


class ICredit(IAmount):

    receipt = TextLine(title=_(u'Receipt number'))


class IIncomeGeneratingActivityCategory(Interface):

    title = TextLine(title=STMessage(u'Title'))


class IIncomeGeneratingActivityCategoryContainer(Interface):

    contains('.IIncomeGeneratingActivityCategory')

    default = Object(
        title=u'The default category',
        schema=IIncomeGeneratingActivityCategory,
        required=False)


class IIncomeGeneratingActivityContainer(Interface):

    contains('.IIncomeGeneratingActivity')


class IIncomeGeneratingActivity(IFeeContainer):

    title = TextLine(title=STMessage(u'Title'))

    description = Text(title=STMessage(u'Description'), required=False)

    category = Choice(
        title=_(u'Category'),
        vocabulary='schooltool.fee.income_generating_activity_categories',
        required=False)

    closed = Bool(title=_(u'Closed'), default=False)

    projected_income = Decimal(title=_(u'Projected income'))

    current_income = Decimal(title=_(u'Current income'))


class IReceiptNumbers(Interface):

    pass
